import keithley20xx
import unittest
import numpy as np
from unittest import TestCase, TestSuite, TextTestRunner
import random
import time
import struct
import PyTango



class KeithleyAcquisitionTest(TestCase):

    def __init__(self, methodName='runTest'):
        super(KeithleyAcquisitionTest, self).__init__(methodName)
        self.keithley2001 = PyTango.DeviceProxy('IDLab/Keithley2001/1')
        self.keithley2010 = PyTango.DeviceProxy('IDLab/Keithley2010/1')

    def runTest(self, result=None):
        raise NotImplementedError('You must implement it')

    def test_gpib_readout_60ms(self, measures_value, input_nplc, input_mode):

        print(self.keithley.identification)
        self.keithley.VoltageAutoRang = 0
        self.keithley.VoltageAutoIntegrationTime = 0
        self.keithley.VoltageAutoDigits = 0
        self.keithley.OutputDataElements = ['read']
        self.keithley.VoltageRang = 1
        self.keithley.AutoZeroEnable = 0
        self.keithley.InitiateContinuous = 0

        # Integration time setting (input_nplc * 0,02 = time in ms)
        self.keithley.VoltageIntegrationTime = input_nplc * 0.02

        # Configure external trigger as source of measures
        self.keithley.SourceMeasureLayer = 'EXT'
        # TODO set TLINK?

        if input_mode == 'buffer':
            # Prepare acquisition
            self.keithley.BufferSize = measures_value
            self.keithley.ClearBuffer()
            self.keithley.BufferElementGroup = 'COMP'
            self.keithley.BufferFeed = 'SENS1'
            self.keithley.BufferControl = 'NEXT'

        if input_mode == 'bus':
            # Prepare acquisition
            self.keithley.CountMeasureLayer = measures_value
            self.keithley.PrepareAcquisition()

        self.keithley.Start()

        loop_flag = True
        first_try = 0

        while loop_flag and first_try < 2:
            time.sleep(300)
            try:
                if input_mode == 'buffer':
                    loop_flag = self.keithley.BufferControl == 'NEV'
                if input_mode == 'bus':
                    loop_flag = measures_value == self.keithley.DataReady
            except:
                first_try += 1

        data = []
        if first_try < 2:
            if input_mode == 'buffer':
                data = self.keithley.BufferData
            if input_mode == 'bus':
                data = self.keithley.RawDataBuffer

            msg = 'Triggers lost:\t' + str(30000 - len(data))
        else:
            msg = 'Communication lost'

        self.assertEqual(30000, len(data), '=' * 80 + '\n' + msg)

    def test_gpib_2001_30ms_30kpoints_bus(self):
        nplc_to_measure = 1.5
        input_value = 30000
        self.keithley = self.keithley2001

        mode = 'bus'
        print('\n' + 'With mode %s taking %f measures with %dms of IT' % (
            mode, input_value, nplc_to_measure*0.02))
        self.test_gpib_readout_60ms(input_value, nplc_to_measure, mode)

    def test_gpib_2001_30ms_30kpoints_buffer(self):
        nplc_to_measure = 1.5
        input_value = 30000
        self.keithley = self.keithley2001

        mode = 'buffer'
        print('\n' + 'With mode %s taking %f measures with %dms of IT' % (
            mode, input_value, nplc_to_measure * 0.02))
        self.test_gpib_readout_60ms(input_value, nplc_to_measure, mode)


if __name__ == '__main__':

    KeithleyTestSuite = TestSuite()

    KeithleyTestSuite.addTest(KeithleyAcquisitionTest(
        'test_gpib_2001_30ms_30kpoints_bus'))

    KeithleyTestSuite.addTest(KeithleyAcquisitionTest(
        'test_gpib_2001_30ms_30kpoints_buffer'))


    runner = TextTestRunner()
    result = runner.run(KeithleyTestSuite)

    errors = len(result.errors)
    failures = len(result.failures)

    print('\n' + '=' * 80 + '\n')
    print('Results ')
    print('Error: \t  %d' % errors)
    print('Failures: %d' % failures)
